@@ -24,22 +24,27 @@
 # DEALINGS IN THE SOFTWARE.
 
 
-import sys
 import os
 import unittest
-import time
 import logging
-import weakref
-
-builddir = os.path.normpath(os.environ["DBUS_TOP_BUILDDIR"])
-pydir = os.path.normpath(os.environ["DBUS_TOP_SRCDIR"])
 
 import dbus
-import _dbus_bindings
-import gobject
 import dbus.glib
 import dbus.service
 
+from dbus._compat import is_py2
+
+try:
+    from tap.runner import TAPTestRunner
+except ImportError:
+    print('1..0 # SKIP cannot import TAPTestRunner')
+    raise SystemExit(0)
+
+try:
+    from gi.repository import GObject as gobject
+except ImportError:
+    print('1..0 # SKIP cannot import GObject')
+    raise SystemExit(0)
 
 logging.basicConfig()
 logging.getLogger().setLevel(1)
@@ -58,33 +63,40 @@ class TestDBusBindings(unittest.TestCase):
         # using dbus.bus.BusConnection!
         conn = dbus.connection.Connection(
                 os.environ['DBUS_SESSION_BUS_ADDRESS'])
+        kwargs = {}
+        if is_py2:
+            kwargs['utf8_strings'] = True
         unique = conn.call_blocking('org.freedesktop.DBus',
                                     '/org/freedesktop/DBus',
                                     'org.freedesktop.DBus', 'Hello',
-                                    '', (), utf8_strings=True)
-        self.assert_(unique.__class__ == dbus.UTF8String, repr(unique))
-        self.assert_(unique.startswith(':'), unique)
+                                    '', (), **kwargs)
+        if is_py2:
+            self.assertTrue(unique.__class__ == dbus.UTF8String, repr(unique))
+        self.assertTrue(unique.startswith(':'), unique)
         conn.set_unique_name(unique)
         return conn, unique
 
     def testCall(self):
         conn, unique = self.get_conn_and_unique()
         ret = conn.call_blocking(NAME, OBJECT, IFACE, 'Echo', 'v', ('V',))
-        self.assertEquals(ret, 'V')
+        self.assertEqual(ret, 'V')
 
     def testCallThroughProxy(self):
         conn, unique = self.get_conn_and_unique()
         proxy = conn.get_object(NAME, OBJECT)
         iface = dbus.Interface(proxy, IFACE)
         ret = iface.Echo('V')
-        self.assertEquals(ret, 'V')
+        self.assertEqual(ret, 'V')
 
     def testSetUniqueName(self):
         conn, unique = self.get_conn_and_unique()
+        kwargs = {}
+        if is_py2:
+            kwargs['utf8_strings'] = True
         ret = conn.call_blocking(NAME, OBJECT, IFACE,
                                  'MethodExtraInfoKeywords', '', (),
-                                 utf8_strings=True)
-        self.assertEquals(ret, (unique, OBJECT, NAME,
+                                 **kwargs)
+        self.assertEqual(ret, (unique, OBJECT, NAME,
                                 'dbus.lowlevel.MethodCallMessage'))
 
 
@@ -92,4 +104,6 @@ if __name__ == '__main__':
     gobject.threads_init()
     dbus.glib.init_threads()
 
-    unittest.main()
+    runner = TAPTestRunner()
+    runner.set_stream(True)
+    unittest.main(testRunner=runner)
